#!/usr/bin/perl -w
#
# parse pxelinux's configfile and generate c-code for menu
#
# 03.03.06 Beat Rubischon <rubischon@phys.ethz.ch>
#
use strict;

my %titles;
my $seq=0;
my %seq2menu;
my %menu2seq;
my %menupos;

my $cdef="";
my $ccode="";

# ---------------
# read configfile
# ---------------
open IN, "../pxelinux.cfg/default";

while(<IN>) {
    chomp;

    if (/^# ::([^:]*)::$/) {
	$titles{'MAIN'}=$1;
    }

    if (/^# (.[^:]*):(.[^:]*):(.[^:]*):(.[^:]*):(.[^:]*)$/) {
	my $menu=$1;
	my $title=$2;
	my $desc=$3;
	my $opt=$4;
	my $target=$5;

	if (! defined $menu2seq{$menu}) {
	    $seq++;
	    $menu2seq{$menu}=$seq;
	    $seq2menu{$seq}=$1;
	}

	if ($4 eq "SUBMENU") {
	    $titles{$target}=$title;
	}

	if (defined $menupos{$menu}) {
	    $menupos{$menu}.="\n"."$title:$desc:$opt:$target";
	} else {
	    $menupos{$menu}="$title:$desc:$opt:$target";
	}
    }
}

close IN;

# -------------
# generate menu
# -------------
$cdef="    char ";
foreach (sort keys %seq2menu) {
    my $menu=$seq2menu{$_};
    if ($_ == 1) {
	$cdef.="$menu";
    } else {
	$cdef.=", $menu";
    }
}
$cdef.=";\n";

foreach (sort {$b <=> $a} keys %seq2menu) {
    my $menu=$seq2menu{$_};
    my $title=$titles{$menu};

    $ccode.="    $menu = add_menu(\" $title \");\n";

    foreach (split(/\n/, $menupos{$menu})) {
	my ($title, $desc, $opt, $target)=split(/:/, $_);

	if ($opt eq "RUN") {
	    $ccode.="    add_item(\"$title\",\"$desc\",OPT_RUN,\"$target\",0);\n";
	} elsif ($opt eq "SUBMENU") {
	    $ccode.="    add_item(\"$title...\",\"$desc\",OPT_SUBMENU,NULL,$target);\n";
	}
    }

    if ($menu ne "MAIN") {
	$ccode.="    add_item(\"Exit this menu\",\"Go one level up\",OPT_EXITMENU,\"exit\",0);\n";
    }

    $ccode.="\n";
}

# -----------------
# fix existing code
# -----------------
my $indef=0;
my $incode=0;

open IN, "pxemenu.src";
open OUT, ">pxemenu.c";

while(<IN>) {
    chomp;

    if (/\/\* END def \*\//) {
	print OUT $cdef;  
	$indef=0;
    }

    if (/\/\* END code \*\//) {
	print OUT $ccode; 
	$incode=0;
    }

    if ($indef == 0 and $incode == 0) {
	print OUT "$_\n";
    }

    if (/\/\* BEGIN def \*\//) {
	$indef=1;
    }

    if (/\/\* BEGIN code \*\//) {
	$incode=1;
    }
}

close IN;
close OUT;
