/*
---------------------------------------------------------------------------
 Generic PXE menu

 - Ask for password
 - If no password is entered, autoboot after 10 seconds
 - If password entered, show a nice menu

 This code will be modified by genmenu.pl - be sure to edit pxemenu.src!
   
 11.01.05 Beat Rubischon <rubischon@phys.ethz.ch>
 03.03.06 Beat Rubischon <rubischon@phys.ethz.ch>
---------------------------------------------------------------------------
*/

#include "menu.h"
#include "biosio.h"
#include "string.h"
#include "syslinux.h"
#include "sha1.h"
#include "passwd.h"

void drawbox(char top, char left, char bot, char right,char attr, char page);
char getch(char *scan);
char toutcnt=0;

/* ----------------------------------------------- */
/* Plot numbers and final timeout after 10 seconds */
/* ----------------------------------------------- */
TIMEOUTCODE ontimeout() {
    char row,col;
    char page = getdisppage();

    getpos(&row,&col,page);

    gotoxy(23, toutcnt * 3 + 1, page);
    csprint(".", 0x74);
    switch(toutcnt) {
      case 0:
        csprint("9", 0x74);
	break;
      case 1:
        csprint("8", 0x74);
	break;
      case 2:
        csprint("7", 0x74);
	break;
      case 3:
        csprint("6", 0x74);
	break;
      case 4:
        csprint("5", 0x74);
	break;
      case 5:
        csprint("4", 0x74);
	break;
      case 6:
        csprint("3", 0x74);
	break;
      case 7:
        csprint("2", 0x74);
	break;
      case 8:
        csprint("1", 0x74);
	break;
      case 9:
        csprint("0", 0x74);
	break;
    }
    csprint(".", 0x74);

    toutcnt++;

    gotoxy(row,col,page);

    if (toutcnt > 10) {
        return CODE_ESCAPE;
    } else {
        return CODE_WAIT;
    }
}

/* ----------------------------- */
/* Ask for password with timeout */
/* ----------------------------- */
void getpass(char *str, unsigned int size)
{
    char c;
    char s;
    char *p = str;
    char page = getdisppage();
    char row,col;
    c = getch(&s);

    while ( c != '\r' && c != '\033' ) {
	switch (c) {
	case '\0':		/* Extended char prefix */
	    break;
	case '\b':
	    if ( p > str ) {
		p--;
		csprint("\b \b",0x17);
	    }
	    break;
	case '\x15':		/* Ctrl-U: kill input */
	    while ( p > str ) {
		p--;
		csprint("\b \b",0x17);
	    }
	    break;
	default:
	    if ( c >= ' ' && (unsigned int)(p-str) < size-1 ) {
	      *p++ = c;
	      getpos(&row,&col,page);
	      csprint("*", 0x17);
	      gotoxy(row,col+1,page);
	    }
	    break;
	}

	c = getch(&s);
    }
    *p = '\0';
    csprint("\r\n",0x17);
}

/* -------------------- */
/* Show password dialog */
/* -------------------- */
int askpass () {
    char page;
    char i, j;
    char bg[2];
    char pw[16];
    SHA1Context sha;
    uint8_t Message_Digest[20];

    toutcnt=0;

    page=getdisppage();

    gotoxy(1, 1, page);
    for (i=0; i < 78; i++)
      csprint(" ", 0x70);

    bg[0]=177;
    bg[1]=0;
    for (i=0; i < 21; i++) {
	gotoxy(i + 2, 1, page);
	for (j=0; j < 78; j++)
	    csprint(bg, 0x01);
    }

    gotoxy(23, 1, page);
    for (i=0; i < 78; i++)
      csprint(" ", 0x74);

    gotoxy(1, 33, page);

    csprint("PXE Boot Menu", 0x70);

    for (i=0; i < 6; i++) {
	gotoxy(i + 3, 2, page);
	for (j=0; j < 43; j++)
	    csprint(" ", 0x17);
	gotoxy(i + 4, 45, page);
	csprint(" ", 0x00);
    }

    gotoxy(9, 3, page);
    for (i=0; i < 42; i++)
    	    csprint(" ", 0x00);

    drawbox(3, 2, 8, 44, 0x17, page);

    gotoxy(4, 4, page);
    csprint("Please wait 10 seconds for autoboot ", 0x17);
    gotoxy(5, 4, page);
    csprint("or enter admin password for maintenance.", 0x17);

    gotoxy(7, 4, page);
    csprint("Enter admin password: ",0x17);
    getpass(pw, sizeof(pw));

    SHA1Reset(&sha);
    SHA1Input(&sha, (char *)pw, strlen(pw));
    SHA1Result(&sha, Message_Digest);
    Message_Digest[20]=0x00;

    if (strcmp(pw, "")) {
	if (strncmp(Message_Digest, PASSWORD, 20)) {
	    return 0;
	} else {
	    return 2;
	}
    } else {
	return 1;
    }
}

/* ------------ */
/* Main routine */
/* ------------ */
int menumain(char *cmdline) {
    char c=0;
    t_menuitem * curr;

/* BEGIN def */
    char MAIN, NET, DISK, MEM, CPU, INSTALL, FC4, S10OSS, DEBIAN, SARGE, ETCH, LIVE, TOOLS;
/* END def */

    (void)cmdline;

    init_menusystem(NULL);
    set_window_size(1,1,24,78);

    // if (syslinux) reg_ontimeout(ontimeout,10,10);
    reg_ontimeout(ontimeout,10,100);

    cls();

    while (c == 0) {
        c=askpass();

	if (c == 1) {
	    cls();
	    if (syslinux) runcommand("local");
	    else csprint("local",0x07);
	    return 0;
	}
    }

    cls();

    reg_ontimeout(NULL,30000L,10);

/* BEGIN code */
    CPU = add_menu(" CPU ");
    add_item("CPU Burn","Stresstest for 586, 686, K6 and K7 CPUs",OPT_RUN,"cpuburn",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    MEM = add_menu(" Memory tests ");
    add_item("Memtest/86+","Memory test",OPT_RUN,"memtest",0);
    add_item("CT Memory Test","Alternate memory test",OPT_RUN,"ctram51",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    DISK = add_menu(" Disk tests ");
    add_item("IBM Drive Fitness Test","Harddisk test for IBM/Hitachi drives",OPT_RUN,"hdtest-ibm",0);
    add_item("SeaTools Diagnostic Suite","Harddisk test for Seagate drives",OPT_RUN,"hdtest-seagate",0);
    add_item("WD Data Liefguard Tools","Harddisk test for WD drives",OPT_RUN,"hdtest-wd",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    NET = add_menu(" NIC setup ");
    add_item("3c90x","3COM Etherlink XL",OPT_RUN,"3com",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    TOOLS = add_menu(" Diagnostics ");
    add_item("NIC setup...","Prepare network cards",OPT_SUBMENU,NULL,NET);
    add_item("Disk tests...","Check disks",OPT_SUBMENU,NULL,DISK);
    add_item("Memory tests...","Burn in memory",OPT_SUBMENU,NULL,MEM);
    add_item("CPU...","Burn cpu",OPT_SUBMENU,NULL,CPU);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    LIVE = add_menu(" Live systems ");
    add_item("ttylinux","Minimalistic Linux",OPT_RUN,"ttylinux",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    ETCH = add_menu(" Etch ");
    add_item("amd64","Manual installation of Debian Etch amd64",OPT_RUN,"debian-etch-amd64",0);
    add_item("i386","Manual installation of Debian Etch i386",OPT_RUN,"debian-etch-i386",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    SARGE = add_menu(" Sarge ");
    add_item("amd64","Manual installation of Debian Sarge amd64",OPT_RUN,"debian-sarge-amd64",0);
    add_item("i386","Manual installation of Debian Sarge i386",OPT_RUN,"debian-sarge-i386",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    DEBIAN = add_menu(" Debian ");
    add_item("Sarge...","Debian Sarge",OPT_SUBMENU,NULL,SARGE);
    add_item("Etch...","Debian Etch",OPT_SUBMENU,NULL,ETCH);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    S10OSS = add_menu(" openSUSE 10.0 ");
    add_item("amd64","Autoyast SuSE 10 OSS amd64",OPT_RUN,"s10oss-amd64",0);
    add_item("i386","Autoyast SuSE 10 OSS i386",OPT_RUN,"s10oss-i386",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    FC4 = add_menu(" Fedora Core 4 ");
    add_item("amd64","Kickstart Fedora Core 4 amd64",OPT_RUN,"fc4-amd64",0);
    add_item("i386","Kickstart Fedora Core 4 i386",OPT_RUN,"fc4-i386",0);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    INSTALL = add_menu(" Install ");
    add_item("Fedora Core 4...","Free RedHat distribution",OPT_SUBMENU,NULL,FC4);
    add_item("openSUSE 10.0...","Free S.u.S.E. distribution",OPT_SUBMENU,NULL,S10OSS);
    add_item("Debian...","Debian GNU Linux",OPT_SUBMENU,NULL,DEBIAN);
    add_item("Exit this menu","Go one level up",OPT_EXITMENU,"exit",0);

    MAIN = add_menu(" PXE Boot Menu ");
    add_item("Boot system","Start with next boot device",OPT_RUN,"local",0);
    add_item("Install...","System installation",OPT_SUBMENU,NULL,INSTALL);
    add_item("Live systems...","Live systems without disk access",OPT_SUBMENU,NULL,LIVE);
    add_item("Diagnostics...","Run diagnostics tools",OPT_SUBMENU,NULL,TOOLS);
    add_item("Boot Mananger","Boot a different device",OPT_RUN,"btmgr",0);
    add_item("Scratch disks","Darik's Boot and Nuke",OPT_RUN,"dban",0);

/* END code */

    curr = showmenus(MAIN);
    if (curr) {
	if (curr->action == OPT_RUN) {
	    if (syslinux) runcommand(curr->data);
	    else csprint(curr->data,0x07);
	    return 1;
	}
	csprint("Error in programming!",0x07);
    }

    if (syslinux) runcommand("local");
    else csprint("local",0x07);

    return 0;
}
